﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Example1
{
    public partial class MainForm : Form
    {
        // variables

        // initial velocity
        private float m_ux, m_uy;

        //current velocity
        private float m_vx, m_vy;

        // initial position
        private PointF m_initialPosition;

        // list of points describing projectile path
        private List<PointF> m_path;

        // index for animation
        private int m_animationIndex;

        // updates the initial velocity [Ux, Uy] from the trackbar controls
        private void UpdateInitialVelocity()
        {
            float angle = (float)(m_angleTrackBar.Value * Math.PI / 180.0);
            float speed = m_speedTrackBar.Value;
            m_ux = (float)(speed * Math.Cos(angle));
            m_uy = -(float)(speed * Math.Sin(angle));

            m_speedLabel.Text = m_speedTrackBar.Value + " units/s";
            m_angleLabel.Text = m_angleTrackBar.Value + " degs";

            m_initialVelocityLabel.Text = string.Format("u = [{0:#.00}  {1:#.00}]", m_ux, m_uy);
            Invalidate();
        }

        public MainForm()
        {
            InitializeComponent();
        }

        // called when form is loaded
        private void OnLoad(object sender, EventArgs e)
        {
            // set initial position to bottom-left of form
            m_initialPosition = new PointF(0.0f, this.ClientRectangle.Height);

            // initialise point path
            m_path = new List<PointF>();

            // update screen settings
            UpdateInitialVelocity();
            OnUpdateDeltaTime(this, EventArgs.Empty);
        }

        // update initial velocity when speed modified
        private void OnSpeedChanged(object sender, EventArgs e)
        {
            UpdateInitialVelocity();
        }

        // update initial velocity when angle modified
        private void OnAngleChanged(object sender, EventArgs e)
        {
            UpdateInitialVelocity();
        }

        // called when Start button clicked
        private void OnStartButton(object sender, EventArgs e)
        {
            // set initial position (p = s)
            float px = m_initialPosition.X, py = m_initialPosition.Y;

            // set initial velocity (v = u)
            m_vx = m_ux; m_vy = m_uy;

            // set acceleration (as g for gravity)
            float g = -100.0f;

            // get delta time in seconds from millisecond trackbar control
            float dt = m_deltaTimeTrackBar.Value / 1000.0f;

            // clear path list and add very first point
            m_path.Clear();
            m_path.Add(m_initialPosition);

            // loop until hitting 'ground'
            while (py <= m_initialPosition.Y)
            {
                // p' = p + v * dt
                px += m_vx * dt;
                py += m_vy * dt;

                // v' = v + a * dt
                m_vy -= g * dt;

                // add new point
                m_path.Add(new PointF(px, py));
            }

            // for redrawing of screen
            Invalidate();

            // configure and enable animation timer
            m_animationTimer.Enabled = false;
            m_animationTimer.Interval = m_deltaTimeTrackBar.Value;
            m_animationIndex = 0;
            m_animationTimer.Enabled = true;
        }

        // called when delta time trackbar is updated
        private void OnUpdateDeltaTime(object sender, EventArgs e)
        {
            m_deltaTimeLabel.Text = m_deltaTimeTrackBar.Value + " ms";
        }

        // called when timer enabled, and at configured intervals
        private void OnAnimationTimer(object sender, EventArgs e)
        {
            if (m_animationIndex < m_path.Count - 1)
                // increment point index if more to go
                ++m_animationIndex;
            else
                // otherwise, terminate animation
                m_animationTimer.Enabled = false;

            // force redrawing for animation
            Invalidate();
        }

        // called whenever form needs to be redrawn
        private void OnPaint(object sender, PaintEventArgs e)
        {
            // get GDI+ graphics object
            Graphics graphics = e.Graphics;

            // determine speed (magnitude of initial velocity vector)
            float speed = m_speedTrackBar.Value;

            // set translation and rotation for aiming arrow
            graphics.TranslateTransform(m_initialPosition.X, m_initialPosition.Y);
            graphics.RotateTransform(-m_angleTrackBar.Value);

            // draw arrow in local coordinates
            graphics.DrawLine(Pens.Green, 0.0f, 0.0f, speed, 0.0f);
            graphics.DrawLine(Pens.Green, speed, 0.0f, speed * 0.9f, speed * -0.05f);
            graphics.DrawLine(Pens.Green, speed, 0.0f, speed * 0.9f, speed * 0.05f);

            // reset transformation for normal drawing
            graphics.ResetTransform();

            // if path set, draw it and also do animtion if timer active
            if (m_path.Count > 0)
            {
                // draw path as a line list
                graphics.DrawLines(Pens.Red, m_path.ToArray());

                // animate projectile if timer active
                if (m_animationTimer.Enabled)
                {
                    // draw black circle for projectile
                    PointF projectilePosition = m_path[m_animationIndex];
                    projectilePosition.X -= 5.0f;
                    projectilePosition.Y -= 5.0f;
                    graphics.FillEllipse(Brushes.Black, new RectangleF(projectilePosition, new SizeF(10, 10)));
                }
            }
        }
    }
}
