﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Example3
{
    // performs collision detection between bodies
    class CollisionDetector
    {
        // returns a collision object if body1 and body2 overlap,
        // or null otherwise
        private static Collision DetectCollision(Body body1, Body body2)
        {
            // compute offset vector of body2 centre with respect to body1 centre
            Vector offset = body2.Position - body1.Position;

            // compute squared distance between centres (avoid square root for now)
            double distanceSquared = offset.LengthSquared;

            // compute sum of body radii (bodies are circles)
            float radiiSum = body1.Radius + body2.Radius;

            // if distance squared greater sum of radii squared, then bodies are not overlapping
            // and hence return null
            // (this is equivalent to comparing distance with sum of radius while avoiding sqrt)
            if (distanceSquared > (double) radiiSum * radiiSum)
                return null;
            
            // if still in method, bodies overlap...

            // compute distance by square root of squared distance
            float distance = (float)Math.Sqrt(distanceSquared);

            // compute penetration as difference between sum of radii and centre distance
            float penetration = radiiSum - distance;

            // compute contact normal by normalizing offset vector of centres
            Vector normal = offset.Normal;

            // compute contact point by moving radius1 units from centre1
            // (and subtracing half penetration to get point in middle of overlap)
            Vector location = body1.Position + (body1.Radius + penetration * 0.5f) * normal;

            // return new collision object with computed details
            return new Collision(body1, body2, location, normal, penetration);
        }

        // detects collisions between all possible pairs from the given list of bodies
        // and returns a list of all collisions (empty list if there are none)
        // NOTE: for N bodies, we have N(N-1)/2 comparisons
        public static List<Collision> DetectCollisions(List<Body> bodies)
        {
            // create new collision list
            List<Collision> collisions = new List<Collision>();

            // process each body in the list
            for (int index1 = 0; index1 < bodies.Count; index1++)
            {
                // get current body
                Body body1 = bodies[index1];

                // for each such body, process all other bodies with greater index
                // (this avoids double comparison of Body A - Body B and Body B - Body A)
                for (int index2 = index1 + 1; index2 < bodies.Count; index2++)
                {
                    // determine second body
                    Body body2 = bodies[index2];

                    // do collision test between the two bodies
                    Collision collision = DetectCollision(body1, body2);

                    // if there is a collision add it to the list
                    if (collision != null)
                        collisions.Add(collision);
                }
            }

            // return collision list
            return collisions;
        }
    }
}
