﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace Example3
{
    // resolves collisions between bodies and between bodies and the window border
    class CollisionHandler
    {
        // resolve the given collision using given restitution coefficient
        private static void HandleCollision(Collision collision, float restitutionCoefficient)
        {
            // get bodies involved in the collision
            Body body1 = collision.Body1;
            Body body2 = collision.Body2;

            // get bodies's mass
            float mass1 = body1.Mass;
            float mass2 = body2.Mass;

            // get collision normal
            Vector normal = collision.Normal;

            // compute relative velocity before collision (Vr)
            Vector relativeVelocity = body2.Velocity - body1.Velocity;

            // compute relative normal speed (Vr . N)
            float relativeNormalSpeed = relativeVelocity * normal;

            // if bodies moving apart, do nothing
            // (this avoids bodies getting stuck - sometimes bodies are still
            // overlapping after a collision is resolved)
            if (relativeNormalSpeed >= 0.0f)
                return;

            // otherwise, bodies are moving into each other, hence colliding...

            // compute simplified equation for impulse scalar j
            // (without inertia tensors and rotation components)
            float impulse = -(1 + restitutionCoefficient) * relativeNormalSpeed * mass1 * mass2
                / (mass1 + mass2);

            // compute vector form of impulses
            // (impulses act along contact normal and oppose each other)
            Vector impulse2 = impulse * normal;
            Vector impulse1 = -impulse2;

            // apply jespective impulses to bodies
            body1.Velocity += impulse1 / mass1;
            body2.Velocity += impulse2 / mass2;

            // solve interpenetration by moving each body half the distance away along
            // contact normal
            float halpPenetration = collision.Penetration * 0.5f;
            body1.Position -= normal * halpPenetration;
            body2.Position += normal * halpPenetration;
        }

        // processes all the collisions given in the list, using the given restitution coeff
        public static void HandleCollisions(List<Collision> collisions, float restitutionCoefficient)
        {
            // process all collisions
            foreach (Collision collision in collisions)
                HandleCollision(collision, restitutionCoefficient);
        }

        // handles body collisions with the window border
        public static void HandleBorderCollisions(List<Body> bodies,
            float restitutionCoefficient, Size windowSize)
        {
            // test each body in the list
            foreach (Body body in bodies)
            {
                // get position and velocity
                Vector position = body.Position;
                Vector velocity = body.Velocity;

                // if body overlaps the left window edge...
                if (position.X - body.Radius < 0.0f)
                {
                    // solve penetration
                    position.X = body.Radius;

                    // apply an impulse to the right
                    velocity.X = Math.Abs(velocity.X * restitutionCoefficient);
                }

                // if body overlaps the right window edge...
                if (position.X + body.Radius >= windowSize.Width)
                {
                    // solve penetration
                    position.X = windowSize.Width - body.Radius;

                    // apply an impulse to the left
                    velocity.X = -Math.Abs(velocity.X * restitutionCoefficient);
                }

                // if body overlaps the top window edge...
                if (position.Y - body.Radius < 0.0f)
                {
                    // solve penetration
                    position.Y = body.Radius;

                    // apply a downward impulse
                    velocity.Y = Math.Abs(velocity.Y * restitutionCoefficient);
                }

                // if body overlaps the bottom window edge...
                if (position.Y + body.Radius >= windowSize.Height)
                {
                    // solve penetration
                    position.Y = windowSize.Height - body.Radius;

                    // apply an upward impulse
                    velocity.Y = -Math.Abs(velocity.Y * restitutionCoefficient);
                }

                // update position and velocity
                body.Position = position;
                body.Velocity = velocity;
            }
        }
    }
}
