﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Example3
{
    public partial class MainForm : Form
    {
        // factor used for mouse dragging force
        private const float DragFactor = 200.0f;

        // physics variables
        private float m_restitutionCoefficient;
        private float m_deltaTime;
        private List<Body> m_bodies;

        // mouse dragging variables
        private Vector m_mouseLocation;
        private Body m_selectedBody;

        public MainForm()
        {
            InitializeComponent();
        }

        // called when the form is loaded
        private void OnLoad(object sender, EventArgs e)
        {
            // set restitution coefficient (e)
            m_restitutionCoefficient = 0.8f;

            // set delta time (Dt) to match timer interval
            m_deltaTime = m_animationTimer.Interval / 1000.0f;

            // initialise a number of bodies with random positions,
            // velocities and sizes
            m_bodies = new List<Body>();
            Random random = new Random();
            for (int index = 0; index < 10; index++)
            {
                Vector position = new Vector(
                    (float)random.NextDouble() * this.ClientSize.Width,
                    (float)random.NextDouble() * this.ClientSize.Height);

                Vector velocity = new Vector(
                    (float)random.NextDouble() * 400.0f - 200.0f,
                    (float)random.NextDouble() * 400.0f - 200.0f);

                float radius = 25.0f + (float)random.NextDouble() * 25.0f;

                m_bodies.Add(new Body(position, velocity, radius));
            }

            m_mouseLocation = new Vector();
        }

        // called at regular intervals for animation
        // NOTE: this is the core of the simulation
        private void OnAnimationTimer(object sender, EventArgs e)
        {
            // update motion of all bodies
            foreach (Body body in m_bodies)
                body.Update(m_deltaTime);

            // determine collisions between bodies
            List<Collision> collisions = CollisionDetector.DetectCollisions(m_bodies);

            // handle body collisions
            CollisionHandler.HandleCollisions(collisions, m_restitutionCoefficient);

            // also handle collision with window border
            CollisionHandler.HandleBorderCollisions(m_bodies, m_restitutionCoefficient, this.ClientSize);

            // force redrawing of form
            Invalidate();
        }

        // called when mouse is pressed
        private void OnMouseDown(object sender, MouseEventArgs e)
        {
            // try to pick a body with the mouse for dragging
            m_selectedBody = null;
            m_mouseLocation.X = e.X;
            m_mouseLocation.Y = e.Y;

            foreach (Body body in m_bodies)
                if (body.ContainsPoint(m_mouseLocation))
                    m_selectedBody = body;
        }

        // called when mouse is moved
        private void OnMouseMove(object sender, MouseEventArgs e)
        {
            m_mouseLocation.X = e.X;
            m_mouseLocation.Y = e.Y;

            // apply pulling force only if a body is selected (dragged)
            if (m_selectedBody != null)
            {
                // if a body is being dragged, generate a force in direction of mouse
                // and proportional to distance (spring-like force)
                Vector force = (m_mouseLocation - m_selectedBody.Position) * DragFactor;

                // apply force to selected body
                m_selectedBody.Force = force;
            }
        }

        // called when mouse button is released
        private void OnMouseUp(object sender, MouseEventArgs e)
        {
            if (m_selectedBody != null)
            {
                m_selectedBody.Force = Vector.Zero;
                m_selectedBody = null;
            }
        }

        // called when form drawing is required
        private void OnPaint(object sender, PaintEventArgs e)
        {
            // get graphics object and set nice smoothing
            Graphics graphics = e.Graphics;
            graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;

            // draw each body
            foreach (Body body in m_bodies)
            {
                RectangleF rectangle = new RectangleF(
                    body.Position.X - body.Radius, body.Position.Y - body.Radius,
                    body.Radius * 2.0f, body.Radius * 2.0f);

                // red for selected body, black otherwise
                Pen pen = body == m_selectedBody ? Pens.Red : Pens.Black;

                // draw body as circle
                graphics.DrawEllipse(pen, rectangle);
            }

            // draw dragging force as line from body centre to mouse (if dragging)
            if (m_selectedBody != null)
                graphics.DrawLine(Pens.Red, m_selectedBody.Position.X, m_selectedBody.Position.Y, m_mouseLocation.X, m_mouseLocation.Y);

            // help message
            graphics.DrawString("Drag bodies with the mouse", this.Font, Brushes.Black, 16.0f, 16.0f);
        }
    }
}
